#!/usr/bin/perl
#==========================================================
# Script Name: ENV v0.1
# File Name  : env.cgi
# Author     : Paul Wilson
# Email      : admin@wiredon.net
# URL        : http://www.wiredon.net/
#==========================================================

    use strict;
    use CGI qw(:standard);

    main();

#==========================================================

sub main {
#----------------------------------------------------------
# Main routine.

    param('pm') ? pm() : param('ps') ? ps() : detect();

}

sub detect {
#----------------------------------------------------------
# Main routine to detect the system environment.

    my $arc                   = {};

# Server.
    $arc->{'SERVER_NAME'}     = $ENV{SERVER_NAME};
    $arc->{'SERVER_ADDR'}     = $ENV{SERVER_ADDR};
    $arc->{'SERVER_TYPE'}     = $^O;
    $arc->{'SERVER_UPTIME'}   = qx/uptime/;
    $arc->{'SERVER_DETAILS'}  = server_details('version');
    $arc->{'SERVER_DETAILS'}  =~ s,#,<BR>#,s;
    $arc->{'SERVER_DETAILS'}  =~ s,\(,<BR>(,s;
    $arc->{'SERVER_CPU'}      = server_details('cpuinfo');
    $arc->{'SERVER_MEM'}      = server_details('meminfo');
    $arc->{'SERVER_MEM'}      =~ s,^.*?<BR>.*?<BR>.*?<BR>,,s;     
    $arc->{'SERVER_PCI'}      = server_details('pci');

# Perl.
    $arc->{'PERL_EXECUTABLE'} = $^X;
    $arc->{'PERL_VERSION'}    = $];
    $arc->{'INC'}             = join("<BR>", @INC);
    $arc->{'INC_ENTRIES'}     = scalar @INC;
    $arc->{'PERL_PATHS'}      = join("<BR>", split(/\s+/, qx/whereis perl/));

# Sendmail.
    $arc->{'SENDMAIL_PATHS'}  = join("<BR>", split(/\s+/, qx/whereis sendmail/));

# Tar and Gzip.
    $arc->{'TAR_PATHS'}       = join("<BR>", split(/\s+/, qx/whereis tar/));
    $arc->{'GZIP_PATHS'}      = join("<BR>", split(/\s+/, qx/whereis gzip/));

# Apache and httpd.
    $arc->{'APACHE_PATHS'}    = join("<BR>", split(/\s+/, qx/whereis apache/));
    $arc->{'HTTPD_PATHS'}     = join("<BR>", split(/\s+/, qx/whereis httpd/));

# Php.
    $arc->{'PHP_PATHS'}       = join("<BR>", split(/\s+/, qx/whereis php/));

# Mysql.
    $arc->{'MYSQL_PATHS'}     = join("<BR>", split(/\s+/, qx/whereis mysql/));

# Man and Perldoc.
    $arc->{'MAN_PATHS'}       = join("<BR>", split(/\s+/, qx/whereis man/));
    $arc->{'PERLDOC_PATHS'}   = join("<BR>", split(/\s+/, qx/whereis perldoc/));

# ENV keys. Used with a LOOP tag.
    %{$arc->{'ENV'}}          = %ENV;

# Script URL.
    $arc->{'SELF'}            = $0;

    print qq{Content-type: text/html\n\n};
    print parse($arc,'system')->{STRING};

}

sub pm {
#----------------------------------------------------------
# Find all installed perl modules.
    
    my $pm = {};

    $pm->{'PERL_MODULES'} = find_modules()->{LIST};

    print qq{Content-type: text/html\n\n};
    print parse($pm,'modules')->{STRING};

}

sub ps {
#----------------------------------------------------------
# List running processes.
    
    my $ps = {};

    $ps->{'PROCESSES'} = qx/ps aux/;

    print qq{Content-type: text/html\n\n};
    print parse($ps,'processes')->{STRING};

}


sub parse {
#----------------------------------------------------------
# Parse the main template.

    open  T, "<./$_[1].html" or error("Could not read $_[1].html: $!");
    read  T, my $string, -s T;
    close T;

# Parse <%LOOP XXX%>.
    $string =~ s/<%\s*LOOP\s+(.*?)\s*%>(.*?)<%\s*ENDLOOP\s*%>/

    my ($loop,$str,$output,$tmp) = ($1,$2,undef,undef);
		
    if (exists $_[0]->{$loop} and ref $_[0]->{$loop} eq 'HASH') { 
	  for (keys %{$_[0]->{$loop}}) {
		 $tmp =  $str;
		 $tmp =~ s|<%\s*KEY\s*%>|$_ ? $_ : "Undetected"|oesg;
		 $tmp =~ s|<%\s*VALUE\s*%>|$_[0]->{$loop}->{$_} ? $_[0]->{$loop}->{$_} : "Undetected"|oesg;
		 $output .= $tmp;
	  }
	  $output;
    }
    else { "Undetected" }
	
    /oesg;

# Parse <%tags%>.
    $string =~ s,<%\s*(.*?)\s*%>,defined $_[0]->{$1} ? $_[0]->{$1} : "Undetected",oesg;

    return { STRING => $string };

}

sub find_modules {
#----------------------------------------------------------
# Locate installed perl modules.

    require File::Find;
	
    my (@pm, %found); 
	
    File::Find::find(sub { /\.pm$/ && /$ARGV[0]/i && push @pm, $File::Find::name }, grep { -r and -d } @INC);
    @pm = grep(!$found{$_}++, @pm); 

    return { LIST => join("<BR>", @pm) }

}

sub server_details {
#----------------------------------------------------------
# Full server details.

    my $f;

    open  P, "</proc/$_[0]" or error("Can't read /proc/$_[0]: $!", 1);
    $f = join("<BR>", <P>);
    close P;

    return $f ? $f : undef;

}

sub error {
#----------------------------------------------------------
# Generic error message.
# We don't need to print a header as it was done in main()

    print qq{Content-type: text/html\n\n} if $_[1];
    print qq{<font face="verdana,arial" size=2>$_[0]</font>};
    exit;

}
    

